/*
Union/Find Algorithm Visualization:  Part of the algoviz@vt collection of algorithm visualizations.
Copyright (C) 2004, 2007  Cris Kania, Cliff Shaffer

This file is part of the Union/Find Algorithm Visualization.

Union/Find Algorithm Visualization is free software: you can redistribute
it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

Union/Find Algorithm Visualization is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Union/Find Algorithm Visualization.  If not, see
<http://www.gnu.org/licenses/>.
*/
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;

public class UFControl extends JComponent implements ActionListener, ChangeListener {
	protected JTextPane messagePane;
	private JPanel panel1;
	public JSpinner spinner1, spinner2;
	private JButton buttonUnion, buttonNext;
	private ButtonGroup radioGroup;
	private JComboBox combo1;
	private JRadioButton radio1, radio2;
	UFVisual visual;
	ParPtrTree tree;
	boolean example1;
	boolean example2;
	private class union
	{
		int a, b;
		boolean comp;
		public union(int aa, int bb)
		{
			a = aa;
			b = bb;
			comp = false;
		}
		public union(int aa, int bb, boolean cc)
		{
			a = aa;
			b = bb;
			comp = cc;
		}
	};
	private int ex1sz, ex2sz;
	private Vector<union> ex1data;
	private Vector<union> ex2data;
	private boolean started;
	private int step;

	public UFControl(ParPtrTree tt, UFVisual vis) 
	{
		tree = tt;
		visual = vis;

		example1 = example2 = false;

		ex1sz = 8;
		ex1data = new Vector<union>();
		ex1data.add(new union(0, 1));
		ex1data.add(new union(2, 3));
		ex1data.add(new union(4, 5));
		ex1data.add(new union(0, 3));
		ex1data.add(new union(4, 6));
		ex1data.add(new union(3, 7));

		ex2sz = 12;
		ex2data = new Vector<union>();
		ex2data.add(new union(10, 2, true));
		ex2data.add(new union(5, 3, true));
		ex2data.add(new union(3, 2, true));
		ex2data.add(new union(2, 6, true));
		ex2data.add(new union(0, 1, true));
		ex2data.add(new union(4, 7, true));
		ex2data.add(new union(8, 9, true));
		ex2data.add(new union(9, 0, true));
		ex2data.add(new union(7, 5, true));
		ex2data.add(new union(7, 1, true));
		ex2data.add(new union(0, 11, true));

		// use a vertical box layout for panes
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// make the top control
		tree.textPane = messagePane = new JTextPane();
		messagePane.setEditable(false);
		tree.doc = messagePane.getDocument();
		add(new JScrollPane(messagePane));

		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
		Style style0 = messagePane.addStyle("Style0", def);
		StyleConstants.setForeground(style0, Color.BLUE);
		StyleConstants.setFontFamily(style0, "Times New Roman" );
		StyleConstants.setFontSize(style0, 13);
		Style style1 = messagePane.addStyle("Style1", def);
		StyleConstants.setForeground(style1, Color.BLACK);
		StyleConstants.setFontFamily(style1, "Times New Roman" );
		StyleConstants.setFontSize(style1, 12);
		Style style2 = messagePane.addStyle("Style2", def);
		StyleConstants.setForeground(style2, new Color(0, 192, 0));
		StyleConstants.setFontFamily(style2, "Times New Roman" );
		StyleConstants.setFontSize(style2, 12);
		Style style3 = messagePane.addStyle("Style3", def);
		StyleConstants.setForeground(style3, new Color(192, 0, 0));
		StyleConstants.setFontFamily(style3, "Times New Roman" );
		StyleConstants.setFontSize(style3, 13);
		Style style4 = messagePane.addStyle("Style4", def);
		StyleConstants.setForeground(style4, new Color(192, 0, 0));
		StyleConstants.setFontFamily(style4, "Times New Roman" );
		StyleConstants.setFontSize(style4, 13);
		StyleConstants.setBold(style4, true);

		// make compression panel
		panel1 = new JPanel();
		panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
		panel1.add(new JLabel("Compression:"));
		radio1 = new JRadioButton("on");
		radio2 = new JRadioButton("off");
		radio2.setSelected(true);
		radioGroup = new ButtonGroup();
		panel1.add(radio1);
		radioGroup.add(radio1);
		panel1.add(radio2);
		radioGroup.add(radio2);
		add(panel1);

		// vertical gap
		add(Box.createVerticalStrut(5));

		// make union panel
		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
		String[] stringList = new String[tree.array.length];
		for (int i = 0; i < tree.array.length; ++i) stringList[i] = "" + tree.array[i].value;
		SpinnerListModel listModel1 = new SpinnerListModel(stringList);
		spinner1 = new JSpinner(listModel1);
		spinner1.addChangeListener(this);
		panel2.add(spinner1);
		SpinnerListModel listModel2 = new SpinnerListModel(stringList);
		spinner2 = new JSpinner(listModel2);
		spinner2.addChangeListener(this);
		panel2.add(spinner2);
		buttonUnion = (JButton)panel2.add(new JButton("Union"));
		buttonUnion.addActionListener(this);
		panel2.add(Box.createHorizontalStrut(5));
		buttonNext = (JButton)panel2.add(new JButton("Next"));
		buttonNext.setEnabled(false);
		buttonNext.addActionListener(this);
		panel2.setMaximumSize(new Dimension(panel2.getMaximumSize().width, panel2.getPreferredSize().height));
		add(panel2);

		// vertical gap
		add(Box.createVerticalStrut(5));

		// make menu panel
		JPanel panel4 = new JPanel();
		panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
		JButton btReset = (JButton)panel4.add(new JButton("Reset"));
		btReset.addActionListener(this);
		String[] setups = {"Example 1", "Example 2", "8 Nodes", "12 Nodes", "16 Nodes", "20 Nodes"};
		combo1 = new JComboBox(setups);
		combo1.setSelectedIndex(2);
		panel4.add(combo1);
		panel4.setMaximumSize(new Dimension(panel4.getMaximumSize().width, panel4.getPreferredSize().height));
		add(panel4);
	}

	public void stateChanged(ChangeEvent e)
	{
		if (e.getSource() == spinner1)
		{
			int i = 0;
			char c = ((String)spinner1.getValue()).charAt(0);
			while (tree.array[i].value != c) ++i;
			tree.unode1 = i;
			if (tree.unode2 == -1)
			{
				i = 0;
				c = ((String)spinner2.getValue()).charAt(0);
				while (tree.array[i].value != c) ++i;
				tree.unode2 = i;
			}
			visual.repaint();
		}
		else if (e.getSource() == spinner2)
		{
			int i = 0;
			char c = ((String)spinner2.getValue()).charAt(0);
			while (tree.array[i].value != c) ++i;
			tree.unode2 = i;
			if (tree.unode1 == -1)
			{
				i = 0;
				c = ((String)spinner1.getValue()).charAt(0);
				while (tree.array[i].value != c) ++i;
				tree.unode1 = i;
			}
			visual.repaint();
		}
	}

	public void allowInput(boolean state)
	{
		if (state)
		{
			buttonNext.setEnabled(false);
			buttonUnion.setEnabled(true);
			radio1.setEnabled(true);
			radio2.setEnabled(true);
			spinner1.setEnabled(true);
			spinner2.setEnabled(true);
		}
		else
		{
			buttonUnion.setEnabled(false);
			radio1.setEnabled(false);
			radio2.setEnabled(false);
			spinner1.setEnabled(false);
			spinner2.setEnabled(false);
			buttonNext.setEnabled(true);
		}
	}

	public void resetgoo()
	{
		example1 = example2 = false;
		started = false;
		step = 0;
		int index = combo1.getSelectedIndex();
		if (index > 1) tree.reset(4 * index);
		else tree.reset((index == 0) ? ex1sz : ex2sz);
		String[] stringList = new String[tree.array.length];
		for (int i = 0; i < tree.array.length; ++i) stringList[i] = "" + tree.array[i].value;
		spinner1.setModel(new SpinnerListModel(stringList));
		spinner2.setModel(new SpinnerListModel(stringList));
		visual.repaint();
		try 
		{
			tree.doc.remove(0, tree.doc.getLength());
		}
		catch (BadLocationException ble)
		{
		}
		if (index > 1) allowInput(true);
		else
		{
			if (index == 0) example1 = true;
			else example2 = true;
			allowInput(false);
			tree.output("Example " + (index + 1) + ": (click Next)\n", 3);
		}
	}

	public void actionPerformed(ActionEvent e) {
		//System.out.println(e.getActionCommand());

		// reset the sim
		if (e.getActionCommand() == "Reset") {
			resetgoo();
		}

		// union between two nodes
		if (e.getActionCommand() == "Union") {
			// fetch indices of nodes to union
			char c1 = ((String)spinner1.getValue()).charAt(0);
			char c2 = ((String)spinner2.getValue()).charAt(0);
			int i1 = -1, i2 = -1;
			for (int i = 0; i < tree.array.length; ++i) {
				if (tree.array[i].value == c1) i1 = i;
				if (tree.array[i].value == c2) i2 = i;
			}
			allowInput(false);
			// is compression enabled?
			tree.compressionOn = radio1.isSelected();
			// union the nodes
			tree.UNION(i1, i2);
			visual.repaint();
		}

		// step the sim
		if (e.getActionCommand() == "Next") {
			if (example1)
			{
				if (started)
				{
					if (tree.step())
					{
						step++;
						started = false;
					}
				}
				if (step == ex1data.size())
				{
					example1 = false;
					allowInput(true);
					tree.output("End of example 1\n", 3);
					visual.repaint();
					return;
				}
				if (!started)
				{
					// enable/disable compression
					union un = (union)ex1data.get(step);
					if (un.comp && radio2.isSelected())
						radio1.setSelected(true);
					else if (!un.comp && radio1.isSelected())
						radio2.setSelected(true);
					tree.compressionOn = un.comp;
					tree.UNION(un.a, un.b);
					started = true;
				}
				visual.repaint();
				return;
			}
			if (example2)
			{
				if (started)
				{
					if (tree.step())
					{
						step++;
						started = false;
					}
				}
				if (step == ex2data.size())
				{
					example2 = false;
					allowInput(true);
					tree.output("End of example 2\n", 3);
					visual.repaint();
					return;
				}
				if (!started)
				{
					// enable/disable compression
					union un = (union)ex2data.get(step);
					if (un.comp && radio2.isSelected())
						radio1.setSelected(true);
					else if (!un.comp && radio1.isSelected())
						radio2.setSelected(true);
					tree.compressionOn = un.comp;
					tree.UNION(un.a, un.b);
					started = true;
				}
				visual.repaint();
				return;
			}
			if (tree.step()) 
			{
				allowInput(true);
			}
			visual.repaint();
		}
	}

	/*public void paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getWidth()-1, getHeight()-1);
		g.setColor(Color.BLACK);
		g.drawRect(0, 0, getWidth()-1, getHeight()-1);
	}*/
}
