/*
Union/Find Algorithm Visualization:  Part of the algoviz@vt collection of algorithm visualizations.
Copyright (C) 2004, 2007  Cris Kania, Cliff Shaffer

This file is part of the Union/Find Algorithm Visualization.

Union/Find Algorithm Visualization is free software: you can redistribute
it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

Union/Find Algorithm Visualization is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Union/Find Algorithm Visualization.  If not, see
<http://www.gnu.org/licenses/>.
*/
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class UFVisual extends JComponent implements ActionListener, MouseListener {
	ParPtrTree tree;
	UFControl control;
	Timer timer;
	//ImageIcon nodeIcon;

	// dimensions of node
	final int nodeW     = 28;
	final int nodeH     = 25;
	final int offset_y  = 18; // from 36 without singles
	final int lvl_drop  = 20; // drop down for overflow of nodes
	final int child_sep = 14;
	final int depth_sep = 20;
	final int sel_ring  = 2;

	final float TRAVEL_TIME = 0.6f;
	final int DELAY_TIME = 30; // milliseconds
	float t = 0;

	static boolean remake;

	// which node for union we selectin
	int select = 0;

	// create the visual; based on tree
	public UFVisual(ParPtrTree tt) {
		tree = tt;
		//nodeIcon = new ImageIcon("node0.png");
		setBackground(Color.GRAY);
		addMouseListener(this);
		timer = new Timer(DELAY_TIME, this);
		update();
		//for (int i = 0; i < tree.array.length; i++) tree.array[i].p0 = new Point(tree.array[i].p1);
	}

	// draw node tree and array
	public void paint(Graphics g) {
		paintNodes(g);
		paintArray(g);
	}

	public void actionPerformed(ActionEvent e) 
	{
		//System.out.println("H");
		t += DELAY_TIME / 1000.0;
		if (t >= TRAVEL_TIME)
		{
			timer.stop();
			for (int i = 0; i < tree.array.length; i++)
			{
				tree.array[i].p = new Point(tree.array[i].p1);
			}
			repaint();
			return;
		}
		float per = (float)Math.sin((((Math.PI / 2.0) / TRAVEL_TIME) * t));
		for (int i = 0; i < tree.array.length; i++)
		{
			int dx = (tree.array[i].p1.x - tree.array[i].p0.x);
			int dy = (tree.array[i].p1.y - tree.array[i].p0.y);
			tree.array[i].p.x = tree.array[i].p0.x + (int)(per * dx);
			tree.array[i].p.y = tree.array[i].p0.y + (int)(per * dy);
		}
		repaint();
	}

	// update node positions
	public void update()
	{
		t = 0;

		// save p as p0
		for (int i = 0; i < tree.array.length; ++i)
			tree.array[i].p0 = new Point(tree.array[i].p);	

		// calculate width and height of row1 and overflow row 2
		int row1Height = 0, row1Width = 0, row2Width = 0;
		Vector<Integer> row1 = new Vector<Integer>();
		Vector<Integer> row2 = new Vector<Integer>();

		// calc the widths for the roots; find max
		int max = 0;
		for (int i = 0; i < tree.roots.size(); ++i)
		{
			calcWidth((ParPtrTree.Node)tree.roots.get(i), child_sep);
			if (((ParPtrTree.Node)tree.roots.get(i)).width > ((ParPtrTree.Node)tree.roots.get(0)).width) max = i;
		}

		// place max, if larger than window width, at start of first row
		if (((ParPtrTree.Node)tree.roots.get(max)).width > getWidth())
		{
			ParPtrTree.Node subroot = (ParPtrTree.Node)tree.roots.get(max);
			row1Width = subroot.width + child_sep;
			row1Height = subroot.height;
			row1.add(new Integer(max));
		}
		else
		{
			max = -1;
		}

		// place rest of sub trees
		for (int i = 0; i < tree.roots.size(); ++i)
		{
			// make node has been placed
			if (i == max) continue;

			ParPtrTree.Node subroot = (ParPtrTree.Node)tree.roots.get(i);
			if (row1Width + subroot.width < getWidth())
			{
				row1Width += subroot.width + child_sep;
				row1Height = Math.max(subroot.height, row1Height);
				row1.add(new Integer(i));
			}
			else
			{
				row2Width += subroot.width + child_sep;
				row2.add(new Integer(i));
			}
		}
		row1Width -= child_sep;
		row2Width -= child_sep;

		// draw the root nodes
		int x = (getWidth()/2) - (row1Width/2);
		int y = offset_y;
		if (max != -1)
		{
			ParPtrTree.Node subroot = (ParPtrTree.Node)tree.roots.get(max);
			int alt_sep = child_sep + ((getWidth() - subroot.width) / (subroot.children.size() - 1));
			calcWidth(subroot, alt_sep);
			x = (getWidth()/2) - (subroot.width/2);
			placeNode(subroot, x, y, alt_sep);
		}
		else
		{
			for (int i = 0; i < row1.size(); ++i) 
			{
				ParPtrTree.Node subroot = (ParPtrTree.Node)tree.roots.get(((Integer)row1.get(i)).intValue());
				placeNode(subroot, x, y, child_sep);
				x += subroot.width + child_sep;
			}
		}
		x = (getWidth()/2) - (row2Width/2);
		if (row1.size() > 0) y += row1Height + lvl_drop;
		for (int i = 0; i < row2.size(); ++i) 
		{
			ParPtrTree.Node subroot = (ParPtrTree.Node)tree.roots.get(((Integer)row2.get(i)).intValue());
			placeNode(subroot, x, y, child_sep);
			x += subroot.width + child_sep;
		}

		timer.start();
	}

	// recursively place an entire node tree
	//   x - left side of node subtree
	//   y - top of node substree
	private void placeNode(ParPtrTree.Node n, int x, int y, int sep) {
		// keep this node's position
		/*if (n.children.size() > 1)
		{
			ParPtrTree.Node first = (ParPtrTree.Node)n.children.get(0);
			ParPtrTree.Node last  = (ParPtrTree.Node)n.children.get(n.children.size()-1);
			int span = first.width/2 + last.width/2 + child_sep;
			for (int i = 1; i < n.children.size()-1; ++i)
				span += ((ParPtrTree.Node)n.children.get(i)).width + child_sep;
			n.p1.x = x + first.width/2 + span/2 - nodeW/2;
		}
		else*/
			n.p1.x = x + n.width/2 - nodeW/2;
		n.p1.y = y;

		// place children
		y += depth_sep + nodeH;
		for (int i = 0; i < n.children.size(); ++i) {
			placeNode((ParPtrTree.Node)n.children.get(i), x, y, sep);
			x += ((ParPtrTree.Node)n.children.get(i)).width + sep;
		}
	}

	// recursively find horizontal width of node subtree
	private int calcWidth(ParPtrTree.Node n, int sep) {
		if (n.children.size() == 0)
		{
			n.width = nodeW;
			n.height = nodeH;
			return n.width;
		}
		n.width = 0;
		n.height = 0;
		for (int i = 0; i < n.children.size(); ++i)
		{
			ParPtrTree.Node child = (ParPtrTree.Node)n.children.get(i);
			n.width += calcWidth(child, sep) + sep;
			if (child.height > n.height) n.height = child.height;
		}
		n.width -= sep;
		n.height += depth_sep + nodeH;
		return n.width;
	}

	public void paintNodes(Graphics g) {
		if (remake)
		{
			update();
			remake = false;
		}

		// draw parent-child lines
		g.setColor(Color.BLACK);
		for (int i = 0; i < tree.roots.size(); ++i) drawLines(g, (ParPtrTree.Node)tree.roots.get(i));

		// draw da nodes
		for (int i = 0; i < tree.array.length; ++i) {
			// bottom left position of node
			int x = tree.array[i].p.x, y = tree.array[i].p.y;

			// draw selection circle
			if (tree.root1 == i && tree.root2 == i) 
			{
				g.setColor(tree.nColor1);
				g.drawArc(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2, 90, 180);
				g.setColor(tree.nColor2);
				g.drawArc(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2, -90, 180);
			} 
			else if (tree.root1 == i) 
			{
				g.setColor(tree.nColor1);
				g.drawOval(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2);
			}
			else if (tree.root2 == i)
			{
				g.setColor(tree.nColor2);
				g.drawOval(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2);
			}
			else if (tree.array[i].path)
			{
				g.setColor(tree.selColor);
				g.drawOval(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2);
			}
			if (tree.array[i].path && tree.root1 == i && tree.root2 != i && tree.stepper == 2)
			{
				g.setColor(tree.selColor);
				g.drawArc(x-sel_ring, y-sel_ring, nodeW+sel_ring*2, nodeH+sel_ring*2, -90, 180);
			}
			if (tree.unode1 == i)
			{
				int xp[] = new int[3], yp[] = new int[3];
				xp[0] = x-sel_ring; xp[1] = x+6-sel_ring; xp[2] = x-sel_ring;
				yp[0] = y-sel_ring; yp[1] = y-sel_ring; yp[2] = y+6-sel_ring;
				g.setColor(tree.nColor1);
				g.fillPolygon(xp, yp, 3);
			}
			else if (tree.unode2 == i)
			{
				int xp[] = new int[3], yp[] = new int[3];
				g.setColor(tree.nColor2);
				xp[0] = x-sel_ring; xp[1] = x+6-sel_ring; xp[2] = x-sel_ring;
				yp[0] = y-sel_ring; yp[1] = y-sel_ring; yp[2] = y+6-sel_ring;
				g.fillPolygon(xp, yp, 3);
				/*xp[0] = x-sel_ring; xp[1] = x+6-sel_ring; xp[2] = x-sel_ring;
				yp[0] = y+sel_ring+nodeH; yp[1] = y+sel_ring+nodeH; yp[2] = y-6+sel_ring+nodeH;
				g.fillPolygon(xp, yp, 3);
				xp[0] = x+sel_ring+nodeW; xp[1] = x-6+sel_ring+nodeW; xp[2] = x+sel_ring+nodeW;
				yp[0] = y-sel_ring; yp[1] = y-sel_ring; yp[2] = y+6-sel_ring;
				g.fillPolygon(xp, yp, 3);
				xp[0] = x+sel_ring+nodeW+1; xp[1] = x-6+sel_ring+nodeW; xp[2] = x+sel_ring+nodeW+1;
				yp[0] = y+sel_ring+nodeH+1; yp[1] = y+sel_ring+nodeH+1; yp[2] = y-6+sel_ring+nodeH;
				g.fillPolygon(xp, yp, 3);*/
			}

			// draw circle background
			g.setColor(new Color(92, 230, 215));
			g.fillOval(x, y, nodeW, nodeH);
			//g.setColor(new Color(123, 255, 255));
			g.setColor(new Color(25, 64, 59));
			g.drawOval(x, y, nodeW, nodeH);
			//g.drawArc(x, y, nodeW, nodeH, 225, 180);
			//nodeIcon.paintIcon(this, g, x, y);

			// draw node value on the circle
			String val_str = "" + tree.array[i].value;
			g.setColor(Color.BLACK);
			g.drawString(val_str, x + nodeW/2 - g.getFontMetrics().charWidth(tree.array[i].value)/2, y + nodeH/2 + g.getFontMetrics().getAscent()/2);
		}
	}

	public void drawLines(Graphics g, ParPtrTree.Node n) 
	{
		int par_x = n.p.x + nodeW/2, par_y = n.p.y + nodeH;
		for (int i = 0; i < n.children.size(); ++i) {
			int x = ((ParPtrTree.Node)n.children.get(i)).p.x + nodeW/2, y = ((ParPtrTree.Node)n.children.get(i)).p.y;
			g.drawLine(par_x, par_y, x, y);
			drawLines(g, (ParPtrTree.Node)n.children.get(i));
		}
	}

	public void paintArray(Graphics g) {
		// calculate drawing constants
		int boxSize = 20, arcWidth = 10, arcHeight = 10, lowSpace = 16;
		int W = boxSize * tree.array.length;
		int H = boxSize * 2;
		int X = (getWidth() - W) / 2;
		int Y = getHeight() - lowSpace - H;

		// create background of array box
		g.setColor(Color.WHITE);
		g.fillRect(X, Y, W, H);

		// make node data light colored divider
		g.setColor(Color.GRAY);
		g.drawLine(X, Y + H/2, X + W, Y + H/2);

		// draw an outline around the array region
		g.setColor(Color.BLACK);
		g.drawRect(X, Y, W, H);

		// get current font and make font for array indices
		Font currFont = g.getFont();
		Font smallFont = new Font(currFont.getName(), Font.ITALIC, 10);

		// draw nodes
		for (int i = 0; i < tree.array.length; ++i) {
			int xOff = X + boxSize * i;

			// make sure default font is back
			g.setFont(currFont);

			// draw the vertical divider between elements
			if (i > 0) g.drawLine(xOff, Y, xOff, Y + H);

			// draw the parent index for this node in top box
			if (tree.array[i].parent == -1)
				g.drawLine(xOff, Y, xOff + boxSize, Y + boxSize);
			else {
				String par_str = (new Integer(tree.array[i].parent)).toString();
				g.drawString(par_str, xOff + boxSize/2 - g.getFontMetrics().stringWidth(par_str)/2, Y + boxSize/2 + g.getFontMetrics().getAscent()/2);
			}

			// draw node value in lower box
			String val_str = "" + tree.array[i].value;
			g.drawString(val_str, xOff + boxSize/2 - g.getFontMetrics().charWidth(tree.array[i].value)/2, Y + H - boxSize/2 + g.getFontMetrics().getAscent()/2);

			// draw array index below the array box
			String i_str = (new Integer(i)).toString();
			g.setFont(smallFont);
			g.drawString(i_str, xOff + boxSize/2 - g.getFontMetrics().stringWidth(i_str)/2, Y + H + g.getFontMetrics().getAscent());
		}
	}

	public void mouseClicked(MouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1 && tree.stepper == -1)
		{
			for (int i = 0; i < tree.array.length; i++)
			{
				int X = tree.array[i].p.x, Y = tree.array[i].p.y;
				if (e.getX() > X && e.getX() < X + nodeW && e.getY() > Y && e.getY() < Y + nodeH)
				{
					//System.out.println("clicked " + tree.array[i].value);
					if (select == 0)
					{
						tree.unode1 = i;
						control.spinner1.setValue("" + tree.array[i].value);
					}
					else
					{
						tree.unode2 = i;
						control.spinner2.setValue("" + tree.array[i].value);
					}
					repaint();
					select = (select == 0) ? 1 : 0;
				}
			}
		}
	}
	public void mouseEntered(MouseEvent e)
	{
	}
	public void mouseExited(MouseEvent e)
	{
	}
	public void mousePressed(MouseEvent e)
	{
	}
	public void mouseReleased(MouseEvent e)
	{
	}
}
